/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogTransform.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "GuiDialogOAC.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogTransform::TFDialogTransform (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Transform Dialog", 
				"Transform: set v for h = h^v", 6, 4), 
		  d_vBoxList (TRUE, 5),
		  d_hBoxTFactor (TRUE, 5),
		  d_hBoxInvert (TRUE, 5),
		  d_hBoxAdjustSea (TRUE, 5),
		  d_adjTFactor (1.5, 1.0, 5.0, 0.01),
		  d_hsTFactor (d_adjTFactor),
		  d_lblTFactor ("Power Factor "),
		  d_lblInvert ("Invert Power Factor"),
		  d_lblAdjustSea ("Adjust Waterlevel"),
		  d_optionsFrame ("Transform Parameters")

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogTransform\n");
	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	sprintf (this->d_winTitle, "Transform: %s", d_HF->getName());
	this->set_title (this->d_winTitle);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjTFactor.value_changed, this, &TFDialogTransform::updatePreviewCallback);
	connect_to_method (d_btnInvert.clicked, this, &TFDialogTransform::updatePreviewCallback);
	connect_to_method (d_btnAdjustSea.clicked, this, &TFDialogTransform::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogTransform::~TFDialogTransform ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogTransform\n");
}


/*
 *  insertOptions: insert the dialogs actual options 
 */
void TFDialogTransform::insertOptions ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogTransform called without Frame label",
		"TFDialogTransform ::setHFobjs");

	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hBoxTFactor.pack_start (d_lblTFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblTFactor.show ();
	d_hsTFactor.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsTFactor.set_digits (2);
	d_hsTFactor.set_draw_value (TRUE);
	d_hBoxTFactor.pack_end (d_hsTFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsTFactor.show ();
	d_vBoxList.pack_start (d_hBoxTFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxTFactor.show ();

	d_hBoxInvert.pack_start (d_lblInvert, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblInvert.show ();
	d_btnInvert.set_active (FALSE);
	connect_to_method (d_btnInvert.clicked, this, &TFDialogTransform::invertCBCallback);
	d_hBoxInvert.pack_end (d_btnInvert, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnInvert.show ();
	d_vBoxList.pack_start (d_hBoxInvert, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxInvert.show ();

	d_hBoxAdjustSea.pack_start (d_lblAdjustSea, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblAdjustSea.show ();
	d_btnAdjustSea.set_active (FALSE);
	d_hBoxAdjustSea.pack_end (d_btnAdjustSea, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_btnAdjustSea.show ();
	d_vBoxList.pack_start (d_hBoxAdjustSea, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxAdjustSea.show ();

	d_optionsFrame.add (d_vBoxList);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_vBoxList.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();

}



void TFDialogTransform::buildDialogWindow ()
{
	this->set_usize (450, 225);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();
	
	this->show ();
}


void TFDialogTransform::invertCBCallback ()
{	
	if (d_btnInvert.get_active())
		d_baseFrame->set_label ("Transform: set v for h = h^(1/v)"); 
	else
		d_baseFrame->set_label ("Transform: set v for h = h^v"); 

	// need to do this otherwise drawing on widget flashes out our redraw
	this->iterateEvents ();

	updatePreviewCallback ();
}


/*
 *  buttonCallbackApply: transform the height field
 */
void TFDialogTransform::buttonCallbackApply ()
{
	bool		adjustSea=d_btnAdjustSea.get_active (),
			invert=d_btnInvert.get_active ();
	float		value=d_adjTFactor.get_value ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogTransform::buttonCallbackApply");

	if (invert)
		d_HFO->transform ((double)1/(double)value, adjustSea);
	else
		d_HFO->transform ((double)value, adjustSea);

	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated. 
 */
void TFDialogTransform::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	bool		adjustSea=d_btnAdjustSea.get_active (),
			invert=d_btnInvert.get_active ();
	float		value=d_adjTFactor.get_value ();

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogFill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogFill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogFill::updatePreviewCallback");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	if (invert)
		d_HFOPreview->transform ((double)1/(double)value, adjustSea);
	else
		d_HFOPreview->transform ((double)value, adjustSea);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}


